// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.calculator.
// 
// smithers.calculator is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.calculator is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.calculator. If not, see <http://www.gnu.org/licenses/>.

package smithers.calculator;

class MethodsArithmetic {
    
    static CalcObject add(CalcObject... args) {
        double re=0, im=0;
        for (int i=0; i<args.length; i++) {
            if (args[i] instanceof CalcDouble) {
                re += ((CalcDouble)args[i]).value;
            } else if (args[i] instanceof CalcComplexDouble) {
                re += ((CalcComplexDouble)args[i]).re;
                im += ((CalcComplexDouble)args[i]).im;
            } else {
                throw new RuntimeException("Not a complex or double value");
            }
        }
        if (im==0) return new CalcDouble(re);
        else return new CalcComplexDouble(re, im);
    }
    
    static CalcObject sub(CalcObject... args) {
        double re=0, im=0;
        for (int i=0; i<args.length; i++) {
            if (i==0) {
                if (args[i] instanceof CalcDouble) {
                    re += ((CalcDouble)args[i]).value;
                } else if (args[i] instanceof CalcComplexDouble) {
                    re += ((CalcComplexDouble)args[i]).re;
                    im += ((CalcComplexDouble)args[i]).im;
                } else {
                    throw new RuntimeException("Not a complex or double value");
                }
            } else {
                if (args[i] instanceof CalcDouble) {
                    re -= ((CalcDouble)args[i]).value;
                } else if (args[i] instanceof CalcComplexDouble) {
                    re -= ((CalcComplexDouble)args[i]).re;
                    im -= ((CalcComplexDouble)args[i]).im;
                } else {
                    throw new RuntimeException("Not a complex or double value");
                }
            }
        }
        if (im==0) return new CalcDouble(re);
        else return new CalcComplexDouble(re, im);
    }
    
    static CalcObject mul(CalcObject... args) {
        double re=1, im=0, tmpre=0;
        for (int i=0; i<args.length; i++) {
            if (args[i] instanceof CalcDouble) {
                re *= ((CalcDouble)args[i]).value;
                im *= ((CalcDouble)args[i]).value;
            } else if (args[i] instanceof CalcComplexDouble) {
                tmpre = re * ((CalcComplexDouble)args[i]).re - im * ((CalcComplexDouble)args[i]).im;
                im = re * ((CalcComplexDouble)args[i]).im + im * ((CalcComplexDouble)args[i]).re;
                re = tmpre;
            } else {
                throw new RuntimeException("Not a complex or double value");
            }
        }
        if (im==0) return new CalcDouble(re);
        else return new CalcComplexDouble(re, im);
    }
    
    static CalcObject div(CalcObject... args) {
        double re=1, im=0, tmpre=0, c=0, d=0, r=0;
        for (int i=0; i<args.length; i++) {
            if (i==0) {
                if (args[i] instanceof CalcDouble) {
                    re = ((CalcDouble)args[i]).value;
                    im = 0;
                } else if (args[i] instanceof CalcComplexDouble) {
                    re = ((CalcComplexDouble)args[i]).re;
                    im = ((CalcComplexDouble)args[i]).im;
                } else {
                    throw new RuntimeException("Not a complex or double value");
                }
            } else {
                if (args[i] instanceof CalcDouble) {
                    re /= ((CalcDouble)args[i]).value;
                    im /= ((CalcDouble)args[i]).value;
                } else if (args[i] instanceof CalcComplexDouble) {
                    c = ((CalcComplexDouble)args[i]).re;
                    d = ((CalcComplexDouble)args[i]).im;
                    r = c*c+d*d;
                    tmpre = (re*c+im*d)/r;
                    im = (im*c-re*d)/r;
                    re = tmpre;
                } else {
                    throw new RuntimeException("Not a complex or double value");
                }
            }
        }
        if (im==0) return new CalcDouble(re);
        else return new CalcComplexDouble(re, im);
    }
    
    static CalcObject pow(CalcObject... args) {
        double value = 1;
        for (int i=args.length-1; i>=0; i--) {
            if (args[i] instanceof CalcDouble) value = Math.pow(((CalcDouble)args[i]).value, value);
            else throw new RuntimeException("Not a double value");
        }
        return new CalcDouble(value);
    }
    
    static CalcObject sqrt(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            double x = ((CalcDouble)args[0]).value;
            if (x>=0) return new CalcDouble(Math.sqrt(x));
            else return new CalcComplexDouble(0, Math.sqrt(-x));
        } else if (args[0] instanceof CalcComplexDouble) {
            double abs = ((CalcComplexDouble)args[0]).abs();
            double arg = ((CalcComplexDouble)args[0]).arg();
            if (arg<0) arg += 2*Math.PI;
            return CalcComplexDouble.fromAbsArg(Math.sqrt(abs), arg/2);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject neg(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(-((CalcDouble)args[0]).value);
        } else if (args[0] instanceof CalcComplexDouble) {
            return new CalcComplexDouble(-((CalcComplexDouble)args[0]).re, -((CalcComplexDouble)args[0]).im);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject inv(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(1/((CalcDouble)args[0]).value);
        } else if (args[0] instanceof CalcComplexDouble) {
            double re = ((CalcComplexDouble)args[0]).re;
            double im = ((CalcComplexDouble)args[0]).im;
            double abss = re*re + im*im;
            return new CalcComplexDouble(re / abss, -im / abss);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
}
