// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.extras.
// 
// smithers.extras is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.extras is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.extras. If not, see <http://www.gnu.org/licenses/>.

package smithers.extras;

/**
 * Class to implement 1-dimensional cellular automata. This is currently a
 * standalone program which runs on the command line. It should be given 3
 * arguments as follows. The first argument is the rule as an integer, as
 * described in Wolfram's A New Kind Of Science. The second is the size of the
 * area to operate on, which will be initialised as all 0s with a 1 in the
 * middle. The third is the number of steps to run.
 */
public class Cellular {
    
    private static void out(int[] x) {
        String s = "";
        for (int i=0; i<x.length; i++) s += x[i]==1 ? "#" : " ";
        System.out.println(s);
    }
    
    /**
     * Runs the automaton.
     * 
     * @param args the arguments
     */
    public static void main(String[] args) {
        int i, j;
        int x1, x2, x3;
        int rule, width, steps;
        
        try {
            rule  = Integer.parseInt(args[0]);
            width = Integer.parseInt(args[1]);
            steps = Integer.parseInt(args[2]);
        } catch (Exception e) {
            System.out.println("smithers.extras.Cellular -- simulates 1-dimensional cellular automata.");
            System.out.println("Usage:  java smithers.extras.Cellular rule width steps");
            return;
        }
        
        int[] cells = new int[width];
        int[] oldcells = new int[width];
        
        for (i=0; i<width; i++) cells[i]=0;
        cells[width/2] = 1;
        
        out(cells);
        
        for (i=0; i<steps; i++) {
            for (j=0; j<width; j++) {
                oldcells[j] = cells[j];
            }
            for (j=0; j<width; j++) {
                x1 = j==0 ? oldcells[width-1] : oldcells[j-1];
                x2 = oldcells[j];
                x3 = j+1==width ? oldcells[0] : oldcells[j+1];
                
                cells[j] = Integer.rotateRight(rule,4*x1+2*x2+x3+1) < 0 ? 1 : 0;
            }
            out(cells);
        }
    }
    
}
