// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.twisty.
// 
// smithers.twisty is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.twisty is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.twisty. If not, see <http://www.gnu.org/licenses/>.

package smithers.twisty;

/**
 * A notation for the Square One puzzle. Follows the notation described at <a
 * href="http://www.jaapsch.net/puzzles/square1.htm#notation">
 * http://www.jaapsch.net/puzzles/square1.htm#notation</a>, with one addition,
 * namely that a backslash '\' may be used to flip the opposite half of the
 * puzzle.
 */
public class SquareOneNotation implements PuzzleNotation<SquareOne> {
    
    /**
     * {@inheritDoc}
     */
    public Twist[] getTwists(String description, SquareOne puzzle) {
        int twistCount=0;
        for (int i=0; i<description.length(); i++) {
            if (description.charAt(i) == '(' || description.charAt(i) == '/' || description.charAt(i) == '\\') twistCount++;
        }
        
        Twist[] result = new Twist[twistCount];
        int t=0;
        int j, k;
        String[] sub;
        int[] amt = new int[2];
        for (int i=0; i<description.length(); i++) {
            if (description.charAt(i) == '(') {
                j = description.indexOf(")", i);
                if (j==-1) throw new RuntimeException("Invalid twist description.");
                sub = description.substring(i+1, j).split(",");
                amt[0] = 0;
                amt[1] = 0;
                try {
                    for(k=0; k<sub.length; k++) amt[k%2] = Integer.parseInt(sub[k]);
                } catch (NumberFormatException e) {
                    throw new RuntimeException("Invalid twist description.", e);
                }
                result[t++] = puzzle.getTurn(amt[0], amt[1]);
                i=j;
            } else if (description.charAt(i) == '/') {
                result[t++] = puzzle.getFlip(0);
            } else if (description.charAt(i) == '\\') {
                result[t++] = puzzle.getFlip(1);
            }
        }
        return result;
    }
    
}
